@@ -27,7 +27,7 @@ solve(const char *equation)
 {
     int ret;
     MPEquationOptions options;
-    MPNumber z;
+    MPNumber z = mp_new();
     gchar *result_str = NULL;
 
     memset(&options, 0, sizeof(options));
@@ -46,6 +46,7 @@ solve(const char *equation)
         printf("%s\n", result_str);
     }
     g_free(result_str);
+    mp_clear(&z);
 }
 
 
@@ -66,10 +67,9 @@ str_adjust(char *str)
 }
 
 int
-main(int argc, char **argv)
+main(void)
 {
     char *equation, *line;
-    size_t nbytes = MAXLINE;
 
     /* Seed random number generator. */
     srand48((long) time((time_t *) 0));
@@ -81,7 +81,7 @@ main(int argc, char **argv)
     equation = (char *) malloc(MAXLINE * sizeof(char));
     while (1) {
         printf("> ");
-        line = fgets(equation, nbytes, stdin);
+        line = fgets(equation, MAXLINE, stdin);
 
         if (line != NULL)
             str_adjust(equation);
