@@ -9,6 +9,10 @@
  * license.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -45,7 +49,7 @@ solve(const char *equation)
 {
     MPEquationOptions options;
     MPErrorCode error;
-    MPNumber result;
+    MPNumber result = mp_new();
     char *result_str;
 
     memset(&options, 0, sizeof(options));
@@ -57,15 +61,18 @@ solve(const char *equation)
     error = mp_equation_parse(equation, &options, &result, NULL);
     if(error == PARSER_ERR_MP) {
         fprintf(stderr, "Error: %s\n", mp_get_error());
+        mp_clear(&result);
         exit(1);
     }
     else if(error != 0) {
         fprintf(stderr, "Error: %s\n", mp_error_code_to_string(error));
+        mp_clear(&result);
         exit(1);
     }
     else {
         result_str = mp_serializer_to_string(mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 9), &result);
         printf("%s\n", result_str);
+        mp_clear(&result);
         exit(0);
     }
 }
@@ -180,7 +187,7 @@ get_options(int argc, char *argv[])
 }
 
 static void
-quit_cb(MathWindow *window)
+quit_cb(MathWindow *win)
 {
     gtk_main_quit();
 }
@@ -190,7 +197,7 @@ int main(int argc, char **argv)
     MathEquation *equation;
     MathButtons *buttons;
     int accuracy = 9, word_size = 64, base = 10;
-    gboolean show_tsep = FALSE, show_zeroes = FALSE;
+    gboolean show_tsep = FALSE, show_zeroes = FALSE, show_hist = FALSE;
     MpDisplayFormat number_format;
     MPAngleUnit angle_units;
     ButtonMode button_mode;
@@ -213,6 +220,7 @@ int main(int argc, char **argv)
     base = g_settings_get_int(g_settings_var, "base");
     show_tsep = g_settings_get_boolean(g_settings_var, "show-thousands");
     show_zeroes = g_settings_get_boolean(g_settings_var, "show-zeroes");
+    show_hist = g_settings_get_boolean(g_settings_var, "show-history");
     number_format = g_settings_get_enum(g_settings_var, "number-format");
     angle_units = g_settings_get_enum(g_settings_var, "angle-units");
     button_mode = g_settings_get_enum(g_settings_var, "button-mode");
@@ -244,6 +252,7 @@ int main(int argc, char **argv)
     window = math_window_new(equation);
     buttons = math_window_get_buttons(window);
     g_signal_connect(G_OBJECT(window), "quit", G_CALLBACK(quit_cb), NULL);
+    math_window_set_show_history(window, show_hist);
     math_buttons_set_programming_base(buttons, base);
     math_buttons_set_mode(buttons, button_mode); // FIXME: We load the basic buttons even if we immediately switch to the next type
 
