@@ -69,9 +69,6 @@ G_DEFINE_TYPE_WITH_PRIVATE (MathButtons, math_buttons, GTK_TYPE_BOX);
 #define GET_WIDGET(ui, name) \
           GTK_WIDGET(gtk_builder_get_object((ui), (name)))
 
-#define WM_WIDTH_FACTOR  10
-#define WM_HEIGHT_FACTOR 30
-
 typedef enum
 {
     NUMBER,
@@ -222,9 +219,6 @@ static ButtonData button_data[] = {
     {"twos_complement",    "twos ", FUNCTION,
       /* Tooltip for the two's complement button */
       N_("Two's Complement")},
-    {"trunc",              "trunc ", FUNCTION,
-      /* Tooltip for the truncate button */
-      N_("Truncate")},
     {"start_group",        "(", GROUP,
       /* Tooltip for the start group button */
       N_("Start Group [(]")},
@@ -370,33 +364,40 @@ update_bit_panel(MathButtons *buttons)
     if (!buttons->priv->bit_panel)
         return;
 
+    x = mp_new();
     enabled = math_equation_get_number(buttons->priv->equation, &x);
 
     if (enabled) {
-        MPNumber max, fraction;
+        MPNumber max = mp_new();
+        MPNumber fraction = mp_new();
 
         mp_set_from_unsigned_integer(G_MAXUINT64, &max);
         mp_fractional_part(&x, &fraction);
         if (mp_is_negative(&x) || mp_is_greater_than(&x, &max) || !mp_is_zero(&fraction))
             enabled = FALSE;
         else
-            bits = mp_cast_to_unsigned_int(&x);
+            bits = mp_to_unsigned_integer(&x);
+        mp_clear(&max);
+        mp_clear(&fraction);
     }
 
     gtk_widget_set_sensitive(buttons->priv->bit_panel, enabled);
     gtk_widget_set_sensitive(buttons->priv->base_label, enabled);
 
     if (!enabled)
+    {
+        mp_clear(&x);
         return;
+    }
 
     for (i = 0; i < MAXBITS; i++) {
-        const gchar *label;
+        const gchar *bin_label;
 
         if (bits & (1LLU << (MAXBITS-i-1)))
-            label = " 1";
+            bin_label = " 1";
         else
-            label = " 0";
-        gtk_label_set_text(GTK_LABEL(buttons->priv->bit_labels[i]), label);
+            bin_label = " 0";
+        gtk_label_set_text(GTK_LABEL(buttons->priv->bit_labels[i]), bin_label);
     }
 
     base = math_equation_get_base(buttons->priv->equation);
@@ -422,6 +423,7 @@ update_bit_panel(MathButtons *buttons)
 
     gtk_label_set_text(GTK_LABEL(buttons->priv->base_label), label->str);
     g_string_free(label, TRUE);
+    mp_clear(&x);
 }
 
 
@@ -901,17 +903,16 @@ shift_left_cb(GtkWidget *widget, MathButtons *buttons)
 
         for (i = 1; i < 16; i++) {
             GtkWidget *item, *label;
-            gchar *format, *text;
+            gchar *text;
 
             if (i < 10) {
                 /* Left Shift Popup: Menu item to shift left by n places (n < 10) */
-                format = ngettext("_%d place", "_%d places", i);
+                text = g_strdup_printf(ngettext("_%d place", "_%d places", i), i);
             }
             else {
                 /* Left Shift Popup: Menu item to shift left by n places (n >= 10) */
-                format = ngettext("%d place", "%d places", i);
+                text = g_strdup_printf(ngettext("%d place", "%d places", i), i);
             }
-            text = g_strdup_printf(format, i);
             label = gtk_label_new_with_mnemonic(text);
 
             item = gtk_menu_item_new();
@@ -944,17 +945,16 @@ shift_right_cb(GtkWidget *widget, MathButtons *buttons)
 
         for (i = 1; i < 16; i++) {
             GtkWidget *item, *label;
-            gchar *format, *text;
+            gchar *text;
 
             if (i < 10) {
                 /* Right Shift Popup: Menu item to shift right by n places (n < 10) */
-                format = ngettext("_%d place", "_%d places", i);
+                text = g_strdup_printf(ngettext("_%d place", "_%d places", i), i);
             }
             else {
                 /* Right Shift Popup: Menu item to shift right by n places (n >= 10) */
-                format = ngettext("%d place", "%d places", i);
+                text = g_strdup_printf(ngettext("%d place", "%d places", i), i);
             }
-            text = g_strdup_printf(format, i);
             label = gtk_label_new_with_mnemonic(text);
 
             item = gtk_menu_item_new();
@@ -1177,6 +1177,7 @@ finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons)
     dialog = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "finc_dialog"));
 
     for (i = 0; i < 4; i++) {
+        arg[i] = mp_new();
         if (finc_dialog_fields[dialog][i] == NULL) {
             continue;
         }
@@ -1187,6 +1188,10 @@ finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons)
     gtk_widget_grab_focus(GET_WIDGET(buttons->priv->financial_ui, finc_dialog_fields[dialog][0]));
 
     do_finc_expression(buttons->priv->equation, dialog, &arg[0], &arg[1], &arg[2], &arg[3]);
+
+    for (i = 0; i < 4; i++) {
+        mp_clear(&arg[i]);
+    }
 }
 
 
@@ -1200,7 +1205,7 @@ character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButto
     text = gtk_entry_get_text(GTK_ENTRY(buttons->priv->character_code_entry));
 
     if (response_id == GTK_RESPONSE_OK) {
-        MPNumber x;
+        MPNumber x = mp_new();
         int i = 0;
 
         mp_set_from_integer(0, &x);
@@ -1213,10 +1218,9 @@ character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButto
             else
                 break;
         }
-
         math_equation_insert_number(buttons->priv->equation, &x);
+        mp_clear(&x);
     }
-
     gtk_widget_hide(dialog);
 }
 
