@@ -49,8 +49,6 @@ enum {
 
 static GType number_mode_type, number_format_type, angle_unit_type;
 
-#define MAX_DIGITS 512
-
 /* Expression mode state */
 typedef struct {
     MPNumber ans;              /* Previously calculated answer */
@@ -290,6 +288,8 @@ reformat_display(MathEquation *equation)
 
     /* Add/remove thousands separators */
     reformat_separators(equation);
+
+    g_signal_emit_by_name(equation, "display-changed");
 }
 
 
@@ -300,6 +300,7 @@ get_current_state(MathEquation *equation)
     gint ans_start = -1, ans_end = -1;
 
     state = g_malloc0(sizeof(MathEquationState));
+    state->ans = mp_new();
 
     if (equation->priv->ans_start)
     {
@@ -939,7 +940,7 @@ math_equation_get_answer(MathEquation *equation)
 void
 math_equation_store(MathEquation *equation, const gchar *name)
 {
-    MPNumber t;
+    MPNumber t = mp_new();
 
     g_return_if_fail(equation != NULL);
     g_return_if_fail(name != NULL);
@@ -948,6 +949,7 @@ math_equation_store(MathEquation *equation, const gchar *name)
         math_equation_set_status(equation, _("No sane value to store"));
     else
         math_variables_set(equation->priv->variables, name, &t);
+    mp_clear(&t);
 }
 
 
@@ -969,16 +971,38 @@ math_equation_set(MathEquation *equation, const gchar *text)
     clear_ans(equation, FALSE);
 }
 
+static void
+math_equation_set_with_history(MathEquation *equation, const gchar *text)
+{
+    MathEquationState *state;
+    MPNumber x = mp_new();
+
+    math_equation_get_number(equation, &x);
+    math_equation_set(equation, text);
+
+    /* Notify history */
+    state = get_current_state(equation);
+    g_signal_emit_by_name(equation, "history", state->expression, &x);
+
+    free_state(state);
+    mp_clear(&x);
+}
+
 
 void
 math_equation_set_number(MathEquation *equation, const MPNumber *x)
 {
     char *text;
     GtkTextIter start, end;
+    MathEquationState *state;
 
     g_return_if_fail(equation != NULL);
     g_return_if_fail(x != NULL);
 
+    /* Notify history */
+    state = get_current_state(equation);
+    g_signal_emit_by_name(equation, "history", state->expression, x);
+
     /* Show the number in the user chosen format */
     text = mp_serializer_to_string(equation->priv->serializer, x);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
@@ -990,7 +1014,11 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
     equation->priv->ans_start = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(equation), NULL, &start, FALSE);
     equation->priv->ans_end = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(equation), NULL, &end, TRUE);
     gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(equation), equation->priv->ans_tag, &start, &end);
+
+    g_object_notify(G_OBJECT(equation), "display");
+
     g_free(text);
+    free_state(state);
 }
 
 
@@ -1143,7 +1171,6 @@ get_variable(const char *name, MPNumber *z, void *data)
         else
             result = 0;
     }
-
     free(lower_name);
 
     return result;
@@ -1198,7 +1225,7 @@ math_equation_solve_real(gpointer data)
     gint n_brackets = 0, result;
     gchar *c, *text, *error_token;
     GString *equation_text;
-    MPNumber z;
+    MPNumber z = mp_new();
 
     text = math_equation_get_equation(equation);
     equation_text = g_string_new(text);
@@ -1222,6 +1249,7 @@ math_equation_solve_real(gpointer data)
     switch (result) {
         case PARSER_ERR_NONE:
             solvedata->number_result = g_slice_new(MPNumber);
+            *solvedata->number_result = mp_new();
             mp_set_from_mp(&z, solvedata->number_result);
             break;
 
@@ -1262,6 +1290,7 @@ math_equation_solve_real(gpointer data)
             break;
     }
     g_async_queue_push(equation->priv->queue, solvedata);
+    mp_clear(&z);
 
     return NULL;
 }
@@ -1297,10 +1326,11 @@ math_equation_look_for_answer(gpointer data)
     }
     else if (result->number_result != NULL) {
         math_equation_set_number(equation, result->number_result);
+        mp_clear(result->number_result);
         g_slice_free(MPNumber, result->number_result);
     }
     else if (result->text_result != NULL) {
-        math_equation_set(equation, result->text_result);
+        math_equation_set_with_history(equation, result->text_result);
         g_free(result->text_result);
     }
     g_slice_free(SolveData, result);
@@ -1342,25 +1372,45 @@ static gpointer
 math_equation_factorize_real(gpointer data)
 {
     GString *text;
-    GList *factors, *factor;
-    MPNumber x;
+    GList *factors, *factor, *next_factor;
+    MPNumber x = mp_new();
     MathEquation *equation = MATH_EQUATION(data);
     SolveData *result = g_slice_new0(SolveData);
+    MpDisplayFormat format = mp_serializer_get_number_format(equation->priv->serializer);
 
+    mp_serializer_set_number_format(equation->priv->serializer, MP_DISPLAY_FORMAT_FIXED);
     math_equation_get_number(equation, &x);
     factors = mp_factorize(&x);
 
     text = g_string_new("");
 
-    for (factor = factors; factor; factor = factor->next) {
+    int e = 1;
+    for (factor = factors; factor != NULL; factor = factor->next)
+    {
         gchar *temp;
         MPNumber *n;
 
         n = factor->data;
+        next_factor = factor->next;
+        if (next_factor != NULL && mp_compare(n, next_factor->data) == 0)
+        {
+            e++;
+            continue;
+        }
         temp = mp_serializer_to_string(equation->priv->serializer, n);
         g_string_append(text, temp);
+
+        if (e > 1)
+        {
+            g_string_append(text, "^");
+            g_string_append_printf(text, "%d", e);
+        }
         if (factor->next)
-            g_string_append(text, "×");
+            g_string_append(text, " × ");
+
+        e = 1;
+
+        mp_clear(n);
         g_slice_free(MPNumber, n);
         g_free(temp);
     }
@@ -1369,6 +1419,9 @@ math_equation_factorize_real(gpointer data)
     result->text_result = g_strndup(text->str, text->len);
     g_async_queue_push(equation->priv->queue, result);
     g_string_free(text, TRUE);
+    mp_clear(&x);
+
+    mp_serializer_set_number_format(equation->priv->serializer, format);
 
     return NULL;
 }
@@ -1377,7 +1430,7 @@ math_equation_factorize_real(gpointer data)
 void
 math_equation_factorize(MathEquation *equation)
 {
-    MPNumber x;
+    MPNumber x = mp_new();
 
     g_return_if_fail(equation != NULL);
 
@@ -1388,9 +1441,10 @@ math_equation_factorize(MathEquation *equation)
     if (!math_equation_get_number(equation, &x) || !mp_is_integer(&x)) {
         /* Error displayed when trying to factorize a non-integer value */
         math_equation_set_status(equation, _("Need an integer to factorize"));
+        mp_clear(&x);
         return;
     }
-
+    mp_clear(&x);
     equation->priv->in_solve = true;
 
     g_thread_new("", math_equation_factorize_real, equation);
@@ -1451,7 +1505,7 @@ math_equation_clear(MathEquation *equation)
 void
 math_equation_shift(MathEquation *equation, gint count)
 {
-    MPNumber z;
+    MPNumber z = mp_new();
 
     g_return_if_fail(equation != NULL);
 
@@ -1465,13 +1519,14 @@ math_equation_shift(MathEquation *equation, gint count)
 
     mp_shift(&z, count, &z);
     math_equation_set_number(equation, &z);
+    mp_clear(&z);
 }
 
 
 void
 math_equation_toggle_bit(MathEquation *equation, guint bit)
 {
-    MPNumber x;
+    MPNumber x = mp_new();
     guint64 bits;
     gboolean result;
 
@@ -1479,18 +1534,20 @@ math_equation_toggle_bit(MathEquation *equation, guint bit)
 
     result = math_equation_get_number(equation, &x);
     if (result) {
-        MPNumber max;
+        MPNumber max = mp_new();
         mp_set_from_unsigned_integer(G_MAXUINT64, &max);
         if (mp_is_negative(&x) || mp_is_greater_than(&x, &max))
             result = FALSE;
         else
-            bits = mp_cast_to_unsigned_int(&x);
+            bits = mp_to_unsigned_integer(&x);
+        mp_clear(&max);
     }
 
     if (!result) {
         math_equation_set_status(equation,
                                  /* Message displayed when cannot toggle bit in display*/
                                  _("Displayed value not an integer"));
+        mp_clear(&x);
         return;
     }
 
@@ -1500,6 +1557,7 @@ math_equation_toggle_bit(MathEquation *equation, guint bit)
 
     // FIXME: Only do this if in ans format, otherwise set text in same format as previous number
     math_equation_set_number(equation, &x);
+    mp_clear(&x);
 }
 
 
@@ -1788,6 +1846,29 @@ math_equation_class_init(MathEquationClass *klass)
                                                         "Serializer",
                                                         MP_TYPE_SERIALIZER,
                                                         G_PARAM_READABLE));
+
+    GType param_types[2] = {G_TYPE_STRING, G_TYPE_POINTER};
+    g_signal_newv("history",
+                  G_TYPE_FROM_CLASS(klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL,
+                  NULL,
+                  NULL,
+                  G_TYPE_NONE,
+                  2,
+                  param_types);
+
+    g_signal_new ("display-changed",
+                  G_TYPE_FROM_CLASS(klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL,
+                  NULL,
+                  NULL,
+                  G_TYPE_NONE,
+                  0,
+                  NULL);
 }
 
 
@@ -1827,8 +1908,8 @@ pre_insert_text_cb(MathEquation  *equation,
         offset = gtk_text_iter_get_offset(location);
         get_ans_offsets(equation, &ans_start, &ans_end);
 
-        /* Inserted inside ans */
-        if (offset > ans_start && offset < ans_end)
+        /* Inserted inside or right before ans */
+        if (offset >= ans_start && offset < ans_end)
             clear_ans(equation, TRUE);
     }
 }
@@ -1941,6 +2022,7 @@ math_equation_init(MathEquation *equation)
 
     equation->priv->variables = math_variables_new();
 
+    equation->priv->state.ans = mp_new();
     equation->priv->state.status = g_strdup("");
     equation->priv->word_size = 32;
     equation->priv->angle_units = MP_DEGREES;
