@@ -77,9 +77,12 @@ static void
 Test(char *expression, char *expected, int expected_error, int trailing_digits)
 {
     MPErrorCode error;
-    MPNumber result;
+    char *error_token = NULL;
+    MPNumber result = mp_new();
 
-    error = mp_equation_parse(expression, &options, &result, NULL);
+    error = mp_equation_parse(expression, &options, &result, &error_token);
+    if (error_token)
+        g_free (error_token);
 
     if (error == 0) {
         char *result_str;
@@ -107,6 +110,7 @@ Test(char *expression, char *expected, int expected_error, int trailing_digits)
             fail("'%s' -> error %s, expected error %s", expression,
                  error_code_to_string(error), error_code_to_string(expected_error));
     }
+    mp_clear(&result);
 }
 
 
@@ -118,7 +122,7 @@ do_convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber
 
 
 static void
-test_conversions()
+test_conversions(void)
 {
     memset(&options, 0, sizeof(options));
     options.base = 10;
@@ -158,6 +162,108 @@ test_conversions()
 }
 
 
+static void try(const char* string, bool result, bool expected)
+{
+    if ((result && !expected) || (!result && expected))
+    {
+        fail("%s -> %s, expected %s", string, expected ? "true" : "false", result ? "true" : "false");
+    }
+    else
+    {
+        pass("%s -> %s", string, result ? "true" : "false");
+    }
+}
+
+
+static void test_mp()
+{
+    MPNumber zero = mp_new();
+    MPNumber one = mp_new();
+    MPNumber minus_one = mp_new();
+    mp_set_from_integer(0, &zero);
+    mp_set_from_integer(1, &one);
+    mp_set_from_integer(-1, &minus_one);
+
+    try("mp_is_zero(-1)", mp_is_zero(&minus_one), false);
+    try("mp_is_zero(0)", mp_is_zero(&zero), true);
+    try("mp_is_zero(1)", mp_is_zero(&one), false);
+
+    try("mp_is_negative(-1)", mp_is_negative(&minus_one), true);
+    try("mp_is_negative(0)", mp_is_negative(&zero), false);
+    try("mp_is_negative(1)", mp_is_negative(&one), false);
+
+    try("mp_is_integer(-1)", mp_is_integer(&minus_one), true);
+    try("mp_is_integer(0)", mp_is_integer(&zero), true);
+    try("mp_is_integer(1)", mp_is_integer(&one), true);
+
+    try("mp_is_positive_integer(-1)", mp_is_positive_integer(&minus_one), false);
+    try("mp_is_positive_integer(0)", mp_is_positive_integer(&zero), true);
+    try("mp_is_positive_integer(1)", mp_is_positive_integer(&one), true);
+
+    try("mp_is_natural(-1)", mp_is_natural(&minus_one), false);
+    try("mp_is_natural(0)", mp_is_natural(&zero), false);
+    try("mp_is_natural(1)", mp_is_natural(&one), true);
+
+    try("mp_is_complex(-1)", mp_is_complex(&minus_one), false);
+    try("mp_is_complex(0)", mp_is_complex(&zero), false);
+    try("mp_is_complex(1)", mp_is_complex(&one), false);
+
+    try("mp_is_equal(-1, -1)", mp_is_equal(&minus_one, &minus_one), true);
+    try("mp_is_equal(-1, 0)", mp_is_equal(&minus_one, &zero), false);
+    try("mp_is_equal(-1, 1)", mp_is_equal(&minus_one, &one), false);
+    try("mp_is_equal(0, -1)", mp_is_equal(&zero, &minus_one), false);
+    try("mp_is_equal(0, 0)", mp_is_equal(&zero, &zero), true);
+    try("mp_is_equal(0, 1)", mp_is_equal(&zero, &one), false);
+    try("mp_is_equal(1, -1)", mp_is_equal(&one, &minus_one), false);
+    try("mp_is_equal(1, 0)", mp_is_equal(&one, &zero), false);
+    try("mp_is_equal(1, 1)", mp_is_equal(&one, &one), true);
+
+    try("mp_is_greater_than(0, -1)", mp_is_greater_than (&zero, &minus_one), true);
+    try("mp_is_greater_than(0, 0)", mp_is_greater_than (&zero, &zero), false);
+    try("mp_is_greater_than(0, 1)", mp_is_greater_than (&zero, &one), false);
+    try("mp_is_greater_than(1, -1)", mp_is_greater_than (&one, &minus_one), true);
+    try("mp_is_greater_than(1, 0)", mp_is_greater_than (&one, &zero), true);
+    try("mp_is_greater_than(1, 1)", mp_is_greater_than (&one, &one), false);
+    try("mp_is_greater_than(-1, -1)", mp_is_greater_than (&minus_one, &minus_one), false);
+    try("mp_is_greater_than(-1, 0)", mp_is_greater_than (&minus_one, &zero), false);
+    try("mp_is_greater_than(-1, 1)", mp_is_greater_than (&minus_one, &one), false);
+
+    try("mp_is_greater_equal(0, -1)", mp_is_greater_equal (&zero, &minus_one), true);
+    try("mp_is_greater_equal(0, 0)", mp_is_greater_equal (&zero, &zero), true);
+    try("mp_is_greater_equal(0, 1)", mp_is_greater_equal (&zero, &one), false);
+    try("mp_is_greater_equal(1, -1)", mp_is_greater_equal (&one, &minus_one), true);
+    try("mp_is_greater_equal(1, 0)", mp_is_greater_equal (&one, &zero), true);
+    try("mp_is_greater_equal(1, 1)", mp_is_greater_equal (&one, &one), true);
+    try("mp_is_greater_equal(-1, -1)", mp_is_greater_equal (&minus_one, &minus_one), true);
+    try("mp_is_greater_equal(-1, 0)", mp_is_greater_equal (&minus_one, &zero), false);
+    try("mp_is_greater_equal(-1, 1)", mp_is_greater_equal (&minus_one, &one), false);
+
+    try("mp_is_less_than(0, -1)", mp_is_less_than (&zero, &minus_one), false);
+    try("mp_is_less_than(0, 0)", mp_is_less_than (&zero, &zero), false);
+    try("mp_is_less_than(0, 1)", mp_is_less_than (&zero, &one), true);
+    try("mp_is_less_than(1, -1)", mp_is_less_than (&one, &minus_one), false);
+    try("mp_is_less_than(1, 0)", mp_is_less_than (&one, &zero), false);
+    try("mp_is_less_than(1, 1)", mp_is_less_than (&one, &one), false);
+    try("mp_is_less_than(-1, -1)", mp_is_less_than (&minus_one, &minus_one), false);
+    try("mp_is_less_than(-1, 0)", mp_is_less_than (&minus_one, &zero), true);
+    try("mp_is_less_than(-1, 1)", mp_is_less_than (&minus_one, &one), true);
+
+    try("mp_is_less_equal(0, -1)", mp_is_less_equal (&zero, &minus_one), false);
+    try("mp_is_less_equal(0, 0)", mp_is_less_equal (&zero, &zero), true);
+    try("mp_is_less_equal(0, 1)", mp_is_less_equal (&zero, &one), true);
+    try("mp_is_less_equal(1, -1)", mp_is_less_equal (&one, &minus_one), false);
+    try("mp_is_less_equal(1, 0)", mp_is_less_equal (&one, &zero), false);
+    try("mp_is_less_equal(1, 1)", mp_is_less_equal (&one, &one), true);
+    try("mp_is_less_equal(-1, -1)", mp_is_less_equal (&minus_one, &minus_one), true);
+    try("mp_is_less_equal(-1, 0)", mp_is_less_equal (&minus_one, &zero), true);
+    try("mp_is_less_equal(-1, 1)", mp_is_less_equal (&minus_one, &one), true);
+
+    mp_clear(&zero);
+    mp_clear(&one);
+    mp_clear(&minus_one);
+}
+
+
 static int
 variable_is_defined(const char *name, void *data)
 {
@@ -186,7 +292,7 @@ set_variable(const char *name, const MPNumber *x, void *data)
 }
 
 static void
-test_equations()
+test_equations(void)
 {
     memset(&options, 0, sizeof(options));
     options.wordlen = 32;
@@ -426,7 +532,7 @@ test_equations()
     test("1!", "1", 0);
     test("5!", "120", 0);
     test("69!", "171122452428141311372468338881272839092270544893520369393648040923257279754140647424000000000000000", 0);
-    test("0.1!", "", PARSER_ERR_MP);
+    test("0.1!", "0.95135077", 0);
     test("−1!", "−1", 0);
     test("(−1)!", "", PARSER_ERR_MP);
     test("−(1!)", "−1", 0);
@@ -435,6 +541,7 @@ test_equations()
     test("2²", "4", 0);
     test("2³", "8", 0);
     test("2¹⁰", "1024", 0);
+    test("2³ ²", "512", 0);
     test("(1+2)²", "9", 0);
     test("(x)²", "4", 0);
     test("|1−3|²", "4", 0);
@@ -471,6 +578,7 @@ test_equations()
     test("2^0.5", "1.414213562", 0);
     test("₃√−8", "−2", 0);
     test("(−8)^(1÷3)", "−2", 0);
+    test("e^-2^-5", "0.969233234", 0);
 
     test("0 mod 7", "0", 0);
     test("6 mod 7", "6", 0);
@@ -478,6 +586,11 @@ test_equations()
     test("8 mod 7", "1", 0);
     test("−1 mod 7", "6", 0);
 
+    test("123^456 mod 78", "27", 0);
+    test("2^2 mod 2", "0", 0);
+    test("1^0 mod 2", "1", 0);
+    test("2^1 mod 1", "0", 0);
+
     test("sgn 0", "0", 0);
     test("sgn 3", "1", 0);
     test("sgn −3", "−1", 0);
@@ -589,6 +702,14 @@ test_equations()
     options.angle_units = MP_GRADIANS;
     test("sin 100", "1", 0);
 
+    test("zeta 0", "−0.5", 0);
+    test("zeta 2", "1.644934067", 0);
+    test("zeta −2", "0", 0);
+
+    test("erf 0", "0", 0);
+    test("erf 1", "0.842700793", 0);
+    test("erf −1", "−0.842700793", 0);
+
     /* Complex numbers */
     options.angle_units = MP_DEGREES;
     test("i", "i", 0);
@@ -664,10 +785,11 @@ test_equations()
 
 
 int
-main (int argc, char **argv)
+main (void)
 {
     setlocale(LC_ALL, "C");
 
+    test_mp();
     test_conversions();
     test_equations();
     if (fails == 0)
